# Copyright (c) 2015 Informatix Inc
# www.piranesi.co.uk

# This software lets you export your model from SketchUp Make 2014+
# to Piranesi 6 Pro

# This script must be edited if you changed the installation folder.
# If you would like help with this, please email info@piranesi.co.uk

# Permission to use, copy, modify, and distribute this ruby script for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

require 'sketchup.rb'

# Utility functions
def windows?
    /mswin/ =~ RUBY_PLATFORM || /mingw/ =~ RUBY_PLATFORM
end

def quote (arg)
    return '"' + arg + '"'
end

# Piranesi and Vedute installation locations - edit if your version of Piranesi was installed elsewhere
def piranesi_install
    if windows?
        # Windows default locations
        install6Pro = 'C:\\Program Files\\Informatix\\Piranesi 6 Pro\\Program\\'
        install6Pro64 = 'C:\\Program Files (x86)\\Informatix\\Piranesi 6 Pro\\Program\\'
        install6ProCommon = 'C:\\Program Files (x86)\\Common Files\\Informatix\\Piranesi 6 Pro\\'

        if (FileTest.exist? install6Pro + 'Vedute.exe') && (FileTest.exist? install6Pro + 'Piranesi.exe')
           return install6Pro
        end
        if (FileTest.exist? install6ProCommon + 'Vedute.exe') && (FileTest.exist? install6Pro + 'Piranesi.exe')
           return install6Pro
        end
        if (FileTest.exist? install6Pro64 + 'Vedute.exe') && (FileTest.exist? install6Pro64 + 'Piranesi.exe')
            return install6Pro64
        end

        UI.messagebox 'Could not find your Piranesi 6 Pro installation.' + "\n" + 'Please install Piranesi or update the location in the epix.rb plugin' + "\n" + 'visit www.piranesi.co.uk for details.'
        fail 'Could not locate Piranesi 6 Pro'

    else
        # Mac OS X default locations
        install6Pro = '/Applications/Piranesi 6 Pro/'

        if FileTest.exist? install6Pro + 'Vedute.app/Contents/MacOS/Vedute'
            return install6Pro
        end

        UI.messagebox 'Could not find your Piranesi 6 Pro installation.' + "\n" + 'Please install Piranesi or update the location in the epix.rb plugin' + "\n" + 'visit www.piranesi.co.uk for details.'
        fail 'Could not locate Piranesi 6 Pro'

    end
    UI.messagebox 'Could not find your Piranesi installation.' + "\n" + 'Please install Piranesi or update the location in the epix.rb plugin'
    fail 'Could not locate Piranesi'
end

def vedute_install
    # Vedute is 32-bit so it's the same folder as Piranesi unless we run Piranesi Pro 64-bit
    if windows?
        # Windows default locations
        install6Pro = 'C:\\Program Files\\Informatix\\Piranesi 6 Pro\\Program\\'
        install6ProCommon = 'C:\\Program Files (x86)\\Common Files\\Informatix\\Piranesi 6 Pro\\'

        if (FileTest.exist? install6ProCommon + 'Vedute.exe') && (FileTest.exist? install6Pro + 'Piranesi.exe')
           return install6ProCommon
        end
    end

    return piranesi_install
end

# Are we using Piranesi Pro? - edit if you have pro and have renamed the installation folder
def pro?
    / Pro/ =~ piranesi_install
end

# Location of Vedute program 
def vedute
    if windows?
        return vedute_install + 'Vedute.exe'
    else
        return vedute_install + 'Vedute.app/Contents/MacOS/Vedute'
    end
end

# Location of Piranesi program
def piranesi
    if windows?
        return piranesi_install + 'Piranesi.exe'
    else
        # MacOS X does not need the Piranesi program
    end
end

# Prompt user for epix file location
def get_epix_file
    # If the model has been saved use its name to initialise the save dialog
    model = Sketchup.active_model
    target=model.path
    if target==""
        file = "Untitled.epx"
        if windows?
            target = Dir.getwd.split('/').join('\\')+'\\'
        else
            target = Dir.getwd + '/'
        end
    else
        file = model.title + ".epx"
        # Strip filename from model path
        if windows?
            target=(target.split('\\')[0..-2]).join('\\') + '\\'
        else
            target=(target.split('/')[0..-2]).join('/') + '/'
        end
    end
    wanted=UI.savepanel('Specify EPix filename', target, file)
    return wanted
end

# Create EPix file from this model using Vedute
def writeEpix
    # Check we know where Vedute and Piranesi are
    veduteexe = vedute
    piranesiexe = piranesi

    # Ask user where to write the epix file
    epix_file = get_epix_file
    if (!epix_file) || (epix_file=="")
        return
    end

    # ensure extension is set correctly
    ext=epix_file.split('.')[-1].downcase
    if ext != 'epx'
        epix_file = epix_file + '.epx'
    end

    unless windows?
        # check for overwrite of png image, OK if already confirmed overwrite of epix
        unless FileTest.exist? epix_file
            png_file = epix_file.split('.')[0..-2].join('.') + '.png'
            if FileTest.exist? png_file
                check = UI.messagebox png_file + ' already exists.' + "\n" + 'Do you want to replace it?', MB_YESNO
                unless 6 == check
                    return
                end
            end
        end
    end

    # prompt for resolution, initialised to current view size
    model = Sketchup.active_model
    view = model.active_view
    height = view.vpheight
    width = view.vpwidth
    results = UI.inputbox ['Width', 'Height'], [width, height], 'Specify epix file size'
    if (!results)
        return
    end

    # Now we are ready to create the epix file

    # Save the sketchup model to a temporary file so we get the current camera etc.
    if windows?
        # note model.save wants / not \\, even on Windows
        tempdir = (ENV['TEMP']).split('\\').join('/')
        tempfile = tempdir + '/to_epix.skp'
    else
        tempfile = '/tmp/to_epix.skp'
    end
    model.save_copy tempfile, Sketchup::Model::VERSION_8

    # Set up command to generate EPix file
    command = [
        quote(veduteexe),
        quote(tempfile),            # model to load
        'DEFAULT',                  # use initial view
        results[0], results[1],     # image size
        quote(epix_file),           # target output file
        '-quit',                    # quit Vedute after creating the file
        '-shadows',                 # include shadows
        '-smooth'                   # turn on smoothing
        ]
    
    # Now run the command
    system(command.join(' '))

    # Save the Sketchup rendering, and on Windows import it into the EPix file
    if windows?
        temppng = tempdir + '/to_epx.png'
        view.write_image temppng, results[0], results[1]
        temptxt = tempdir + '/to_epx.txt'
        temptxtfile = File.open(temptxt, 'w')
        eol = "\n"
        temptxtfile.print 'import ' + quote(temppng) + eol
        temptxtfile.print 'rerender' + eol
        if (pro?)
            temptxtfile.print 'export ' + quote(epix_file) + eol
            temptxtfile.print 'close' + eol
            temptxtfile.print 'open ' + quote(epix_file) + eol
        end
        temptxtfile.close

        command2 = [
        'start',
        quote('Piranesi'),
        quote(piranesiexe),
        '/script',
        quote(temptxt),             # import and rerender
        quote(epix_file)            # epix file we created
        ]

        system(command2.join(' '))
    else
        png_file = epix_file.split('.')[0..-2].join('.') + '.png'
        view.write_image png_file, results[0], results[1]
    end
end

#-----------------------------------------------------------------------------

# Add a 'Write EPix' item to the file menu
if( not file_loaded?('epix.rb') )
    add_separator_to_menu('File')
    UI.menu('File').add_item('Write Piranesi EPix') {writeEpix}
end
file_loaded('epix.rb')
